﻿Imports System.IO
Imports System.Text
'Imports System.Runtime.InteropServices

' Create a setting named ProductKey of type UInteger.

Public Class Form1
    <Flags()> _
    Public Enum FileSystemFeature As UInteger
        ' The file system supports case-sensitive file names.
        CaseSensitiveSearch = 1
        ' The file system preserves the case of file names when it places a name on disk.
        CasePreservedNames = 2
        ' The file system supports Unicode in file names as they appear on disk.
        UnicodeOnDisk = 4
        ' The file system preserves and enforces access control lists (ACL).
        PersistentACLS = 8
        ' The file system supports file-based compression.
        FileCompression = &H10
        ' The file system supports disk quotas.
        VolumeQuotas = &H20
        ' The file system supports sparse files.
        SupportsSparseFiles = &H40
        ' The file system supports re-parse points.
        SupportsReparsePoints = &H80
        ' The specified volume is a compressed volume for example a DoubleSpace volume.
        VolumeIsCompressed = &H8000
        ' The file system supports object identifiers.
        SupportsObjectIDs = &H10000
        ' The file system supports the Encrypted File System (EFS).
        SupportsEncryption = &H20000
        ' The file system supports named streams.
        NamedStreams = &H40000
        ' The specified volume is read-only.
        ReadOnlyVolume = &H80000
        ' The volume supports a single sequential write.
        SequentialWriteOnce = &H100000
        ' The volume supports transactions.
        SupportsTransactions = &H200000
    End Enum

    Private Declare Auto Function GetVolumeInformation Lib "kernel32.dll" ( _
        ByVal RootPathName As String, _
        ByVal VolumeNameBuffer As System.Text.StringBuilder, _
        ByVal VolumeNameSize As UInt32, _
        ByRef VolumeSerialNumber As UInt32, _
        ByRef MaximumComponentLength As UInt32, _
        ByRef FileSystemFlags As FileSystemFeature, _
        ByVal FileSystemNameBuffer As System.Text.StringBuilder, _
        ByVal FileSystemNameSize As UInt32) As UInt32

    ' If the program isn't registered, exit.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' An arbitrary number to identify this program.
        Const program_id As UInt32 = 2267918298

        If (Not IsRegistered(program_id, False)) Then Me.Close()
    End Sub

    ' Return true if the program is properly registered.
    Private Function IsRegistered(ByVal program_id As UInt32, ByVal default_value As Boolean) As Boolean
        Dim volume_name As New StringBuilder(1024)
        Dim file_system_name As New StringBuilder(1024)
        Dim serial_number, max_component_length As UInt32
        Dim file_system_flags As FileSystemFeature

        ' Get the startup directory's drive letter.
        ' Get the drive where the program is running.
        Dim file_info As New FileInfo(Application.StartupPath)
        Dim drive_letter As String = file_info.Directory.Root.Name

        ' Get the information. If we fail, return the default value.
        If (GetVolumeInformation(drive_letter, volume_name, _
            CUInt(volume_name.Capacity), serial_number, max_component_length, _
            file_system_flags, file_system_name, CUInt(file_system_name.Capacity)) = 0) _
        Then
            Return default_value
        End If

        ' Encrypt the serial number to get the product key.
        Dim product_key As UInt32 = Encrypt(program_id, serial_number)

        ' If this matches the saved product key, then the program is registered.
        If (My.Settings.ProductKey = product_key) Then Return True

        ' It's not registered properly.
        ' Display the registration form.
        Dim frm As New RegistrationForm()
        frm.txtProductNumber.Text = serial_number.ToString()
        If (frm.ShowDialog() = DialogResult.Cancel) Then Return False

        ' See if the product key matches.
        Dim entered_key As UInt32 = 0
        Try
            entered_key = UInt32.Parse(frm.txtProductKey.Text)
        Catch
        End Try
        If (entered_key = product_key) Then
            My.Settings.ProductKey = entered_key
            My.Settings.Save()
            Return True
        End If

        ' No match. Give up.
        MessageBox.Show("Incorrect product key.", "Invalid Key", _
                MessageBoxButtons.OK, MessageBoxIcon.Error)
        Return False
    End Function

    ' Simple encryption and decryption.
    Private Function Encrypt(ByVal seed As UInt32, ByVal value As UInt32) As UInt32
        Dim rand As New Random(CInt(seed \ 2))
        Return (value Xor CUInt(UInt32.MaxValue * rand.NextDouble()))
    End Function
End Class
